; https://it-howto.info/reg-file-inport-to-all-users
; IT HowTO © 2018

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; get_files() returns the array of files.         ;;
;; uses command line argunets or open file dialog  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Func get_files()
	if $CmdLine[0] = 0 Then
		Local $sFileOpenDialog = FileOpenDialog('Select one or more .reg files', @ScriptDir& "\", "Reg files (*.reg)", $FD_FILEMUSTEXIST + $FD_MULTISELECT)
		if $sFileOpenDialog = '' Then
			MsgBox(48,'reg file required','No file selected')
			Exit
		EndIf

		$files = StringSplit ( $sFileOpenDialog, "|")
		_ArrayDelete ($files,0)
		if UBound( $files ) > 1 Then
			FileChangeDir($files[0]) ; if more than 1 file selected going to it's directory
			_ArrayDelete ($files,0) ; trimming unnecessary array items (count, folder)
		EndIf
	Else
		Local $files[0]
		For $param In $CmdLine
			if $param = $CmdLine[0] Then ContinueLoop
			if StringRight ( $param, 4 ) <> '.reg' Then ContinueLoop
			if FileExists($param) Then _ArrayAdd ( $files, $param)
		Next
	EndIf
	if UBound( $files ) < 1 Then
		MsgBox(48,'reg file required','No file selected')
		Exit
	EndIf
	Return $files
EndFunc


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; regread_expanded($key, $value)                     ;;
;; returns registry value with expanded dos variables ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Func regread_expanded($key, $value)
	$value = RegRead($key,$value)
	$dos = Run(@ComSpec&' /c echo ' & $value,'',@SW_HIDE,2)
	while 1
		$value=StdoutRead($dos)
		if $value Then ExitLoop
	WEnd
	$value=StringRegExpReplace($value,@CRLF,'')
	Return $value
EndFunc



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; get_profile_paths() returns the array of user profile paths ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Func get_profile_paths()
	$regprofileskey = 'HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\ProfileList'
	Local $profile_paths[0]
	$i = 1
	$profiles = regread_expanded($regprofileskey,'ProfilesDirectory')
	if FileExists( $profiles & '\Default' ) Then
		$defpath = $profiles & '\Default'
	Else
		$defpath = $profiles & '\Default User'
	EndIf
	_ArrayAdd ( $profile_paths, $defpath ) 	; $defpath - path to default user profile became element 0 of the $profile_paths array
	While 1
		$subkey = RegEnumKey($regprofileskey,$i)
		If @error <> 0 then	ExitLoop
		$i+=1
		if StringRegExp( $subkey, '^S-1-5-21-' ) Then
			$ProfileImagePath = regread_expanded( $regprofileskey & '\' & $subkey, 'ProfileImagePath' )
			_ArrayAdd ( $profile_paths, $ProfileImagePath )
		EndIf
	WEnd
	Return $profile_paths
EndFunc


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; import_reg_offline($regfile,$NTUSER_DAT) ;;
;; imports a reg file to registry .dat file ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Func import_reg_offline($regfile,$NTUSER_DAT)
	$success = RunWait('reg load HKU\Custom "' & $NTUSER_DAT & '"','',@SW_HIDE)
	if $success <> 0 Then Return ; we'll exit if error

	; Creating a temporary reg file
	local $reg_filename_only = _PathSplit($regfile,'','','','')
	$regfile_tmp=@TempDir & '\' & $reg_filename_only[3] & '-tmp.reg'
	$reg_content = FileRead($regfile)
	$reg_content = StringReplace( $reg_content ,'[HKEY_CURRENT_USER', '[HKEY_USERS\Custom' )
	$reg_content = StringReplace( $reg_content ,'[-HKEY_CURRENT_USER', '[-HKEY_USERS\Custom' )
	FileWrite( $regfile_tmp, $reg_content )

	RunWait('reg import "' & $regfile_tmp & '"','',@SW_HIDE)
	RunWait('reg unload HKU\Custom','',@SW_HIDE)
	FileDelete($regfile_tmp)
EndFunc

